/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_MappingNotify	1
char    *TestName = "MappingNotify";



#define	EVENT		MappingNotify

int 	tet_thistest;

static void t001(){

Display	*display = Dsp;
Display	*client2;
XEvent	event_return;
int	nmap;
int	count;
unsigned char *map;
unsigned char buf[2];
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion MappingNotify-1.(A)");
	report_assertion("When a MappingNotify event is generated, a MappingNotify");
	report_assertion("event is delivered to all clients.");

	report_strategy("Create client2.");
	report_strategy("Generate MappingNotify event.");
	report_strategy("Verify that MappingNotify event was delivered.");
	report_strategy("Verify members in delivered event structure.");
	report_strategy("Verify that MappingNotify event was delivered to client2.");
	report_strategy("Verify members in delivered event structure.");

	tpstartup();
/* Create client2. */
	if ((client2 = opendisplay()) == (Display *) NULL) {
		delete("Couldn't create client2.");
		return;
	}
	else
		CHECK;
/* Generate MappingNotify event. */
	nmap = XGetPointerMapping(display, buf, sizeof(buf));
	if (nmap < 1) {
		report("XGetPointerMapping returned %d, expected greater than 0", nmap);
		delete("Can't determine size of pointer mapping buffer");
		return;
	}
	else
		CHECK;
	map = (unsigned char *) malloc((unsigned int) (nmap * sizeof(*map)));
	if (map == (unsigned char *) NULL) {
		report("Couldn't allocate %d bytes", nmap * sizeof(*map));
		delete("Memory allocation error");
		return;
	}
	else
		CHECK;
	count = XGetPointerMapping(display, map, nmap);
	if (count < nmap) {
		report("XGetPointerMapping returned %d, expected %d",
			count, nmap);
		delete("XGetPointerMapping inconsistencies");
		free(map);
		return;
	}
	else
		CHECK;
	XSync(display, True);
	XSync(client2, True);
	count = XSetPointerMapping(display, map, nmap);
	if (count != Success) {
		report("XSetPointerMapping returned %d, expected %d",
			count, Success);
		delete("XSetPointerMapping failed");
		free(map);
		return;
	}
	else
		CHECK;
	XSync(display, False);
	XSync(client2, False);
/* Verify that MappingNotify event was delivered. */
	if (!XCheckTypedEvent(display, EVENT, &event_return)) {
		report("No events generated, expected %s event",
			eventname(EVENT));
		FAIL;
	}
	else {
		XMappingEvent good;

/* Verify members in delivered event structure. */
		good = event_return.xmapping;
		good.type = EVENT;
		good.request = MappingPointer;
		if (checkevent((XEvent *) &good, &event_return)) {
			report("Unexpected values in delivered event");
			FAIL;
		}
		else
			CHECK;
	}
/* Verify that MappingNotify event was delivered to client2. */
	if (!XCheckTypedEvent(client2, EVENT, &event_return)) {
		report("No events generated, expected %s event with client2",
			eventname(EVENT));
		FAIL;
	}
	else {
		XMappingEvent good;

/* Verify members in delivered event structure. */
		good = event_return.xmapping;
		good.type = EVENT;
		good.request = MappingPointer;
		if (checkevent((XEvent *) &good, &event_return)) {
			report("Unexpected values in delivered event with client2");
			FAIL;
		}
		else
			CHECK;
	}
	free(map);

	CHECKPASS(7);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
