/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) Applied Testing and Technology, Inc. 1995
* All Rights Reserved.
* 
* 
* Portions of this software are based on Xlib and X Protocol Test Suite.
* We have used this material under the terms of its copyright, which grants
* free use, subject to the conditions below.  Note however that those
* portions of this software that are based on the original Test Suite have
* been significantly revised and that all such revisions are copyright (c)
* 1995 Applied Testing and Technology, Inc.  Insomuch as the proprietary
* revisions cannot be separated from the freely copyable material, the net
* result is that use of this software is governed by the ApTest copyright.
* 
* Copyright (c) 1990, 1991  X Consortium
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy
* of this software and associated documentation files (the "Software"), to deal
* in the Software without restriction, including without limitation the rights
* to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
* copies of the Software, and to permit persons to whom the Software is
* furnished to do so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in
* all copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
* X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
* AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
* CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
* 
* Except as contained in this notice, the name of the X Consortium shall not be
* used in advertising or otherwise to promote the sale, use or other dealings
* in this Software without prior written authorization from the X Consortium.
* 
* Permission to use, copy, modify, distribute, and sell this software and
* its documentation for any purpose is hereby granted without fee,
* provided that the above copyright notice appear in all copies and that
* both that copyright notice and this permission notice appear in
* supporting documentation, and that the name of UniSoft not be
* used in advertising or publicity pertaining to distribution of the
* software without specific, written prior permission.  UniSoft
* makes no representations about the suitability of this software for any
* purpose.  It is provided "as is" without express or implied warranty.
* 
*/
/*
 * SYNOPSIS:
 *   KeySym
 *   XKeycodeToKeysym(display, keycode, idx)
 *   Display	*display;
 *   KeyCode	keycode;
 *   int	idx;
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#include	"xtest.h"
#include	"X11/Xlib.h"
#include	"X11/Xutil.h"
#include	"X11/Xresource.h"
#include	"X11/keysym.h"
#include	"tet_api.h"
#include	"xtestlib.h"
#include	"pixval.h"
#ifdef INPUTEXTENSION
#include        "X11/extensions/XInput.h"
#include        "XItest.h"
#endif

extern	Display	*Dsp;
extern	Window	Win;

extern	Window	ErrdefWindow;
extern	Drawable ErrdefDrawable;
extern	GC		ErrdefGC;
extern	Colormap ErrdefColormap;
extern	Pixmap	ErrdefPixmap;
extern	Atom	ErrdefAtom;
extern	Cursor	ErrdefCursor;
extern	Font	ErrdefFont;


#define T_XKeycodeToKeysym	1
char    *TestName = "XKeycodeToKeysym";

/*
 * Defines for different argument types
 */
#define A_DISPLAY display


/*
 * Arguments to the XKeycodeToKeysym function
 */
static Display	*display;
static KeyCode	keycode;
static int	idx;


#define XK_LATIN1
#include	"X11/keysymdef.h"
#undef XK_LATIN1

int 	tet_thistest;

/*
 * Called at the beginning of each test purpose to reset the
 * arguments to their initial values
 */
static void
setargs()
{
	display = Dsp;
	keycode = 0;
	idx = 0;
}

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

KeyCode	kc;
KeySym	ks;
int 	pass = 0, fail = 0;

 	report_purpose(1);

	report_assertion("Assertion XKeycodeToKeysym-1.(A)");
	report_assertion("A call to XKeycodeToKeysym returns the KeySym specified by");
	report_assertion("element index of the KeyCode vector specified by the keycode");
	report_assertion("argument.");

	report_strategy("Obtain the KeyCode corresponding to KeySym XK_a using XKeysymToKeycode.");
	report_strategy("Obtain the KeySym corresponding to the KeyCode using XKeycodeToKeysym with index = 0.");
	report_strategy("Verify that the KeySym returned is XK_a.");
	report_strategy("Obtain the KeySym corresponding to the KeyCode using XKeycodeToKeysym with index = 1.");
	report_strategy("Verify that the KeySym returned is XK_A.");

	tpstartup();
	setargs();
	kc = XKeysymToKeycode(display, XK_a);

	if(kc == 0) {
		delete("XKeysymToKeycode() returned 0 for KeySym XK_a");
		return;
	} else
		CHECK;

	keycode = kc;
	idx = 0;
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, idx);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_a) {
		report("%s() returned KeySym %lu instead of %lu for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) XK_a, (long) kc, idx);
		FAIL;
	} else
		CHECK;


	idx = 1;
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, idx);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != XK_A) {
		report("%s() returned KeySym %lu instead of %lu for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) XK_A, (long) kc, idx);
		FAIL;
	} else
		CHECK;

	CHECKPASS(3);

	tpcleanup();
	pfcount(pass, fail);
}

static void t002(){

int	maxi,mini;
KeyCode	maxkc, minkc;
KeySym	ks;
int 	pass = 0, fail = 0;

 	report_purpose(2);

	report_assertion("Assertion XKeycodeToKeysym-2.(A)");
	report_assertion("When the symbol specified by element index of the KeyCode");
	report_assertion("vector specified by the keycode argument is not defined,");
	report_assertion("then a call to XKeycodeToKeysym returns NoSymbol.");

	report_strategy("Obtain the maximum keycode using XDisplayKeycodes.");
	report_strategy("Obtain the KeySym corresponding to the maximum KeyCode+1 using XKeycodeToKeysym with index 0.");
	report_strategy("Verify that the call returned NoSymbol.");
	report_strategy("Obtain the KeySym corresponding to the minimum KeyCode using XKeycodeToKeysym with index 9.");
	report_strategy("Verify that the call returned NoSymbol.");

	tpstartup();
	setargs();
	XDisplayKeycodes(display, &mini, &maxi);
	minkc = (KeyCode)mini;
	maxkc = (KeyCode)maxi;

	idx = 0;
	keycode = maxkc + 1;
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, idx);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != NoSymbol) {
		report("%s() returned KeySym value %ul instead of NoSymbol (%lu) for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) NoSymbol, (long) keycode, idx);
		FAIL;
	} else
		CHECK;

	keycode = minkc;
	idx = 9; /* only 0-8 are valid. */
	startcall(display);
	if (isdeleted())
		return;
	ks = XKeycodeToKeysym(display, keycode, idx);
	endcall(display);
	if (geterr() != Success) {
		report("Got %s, Expecting Success", errorname(geterr()));
		FAIL;
	}

	if(ks != NoSymbol) {
		report("%s() returned KeySym value %ul instead of NoSymbol (%lu) for KeyCode %lu and index %d.",
			TestName, (long) ks, (long) NoSymbol, (long) keycode, idx);
		FAIL;
	} else
		CHECK;

	CHECKPASS(2);
	tpcleanup();
	pfcount(pass, fail);
}

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ t002, 2 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = startup;
void	(*tet_cleanup)() = cleanup;
