/*
* Copyright (c) 2005 X.Org Foundation LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) Applied Testing and Technology, Inc. 1993, 1994, 1995
* Copyright (c) 88open Consortium, Ltd. 1990, 1991, 1992, 1993
* All Rights Reserved.
* 
*/
/*
 * SYNOPSIS:
 *   Boolean
 *   XtCvtStringToVisual()
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <XtTest.h>

/*error messages created here*/
char	ebuf[4096];
/*allows a test to extend how long it runs*/
int	xt_tomultiple = 1;

#include <X11/IntrinsicP.h>
#include <X11/ConstrainP.h>
#include <X11/CoreP.h>

XtAppContext app_ctext;
Widget topLevel, panedw, boxw1, boxw2;
Widget labelw, rowcolw, click_quit;

void XtWMH_Proc(str1, str2, str3, str4, str5, car)
String str1, str2, str3, str4, *str5;
Cardinal *car;
{
	avs_set_event(1,1);
}


#define T_XtCvtStringToVisual	1
char    *TestName = "XtCvtStringToVisual";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

#if XT_X_RELEASE > 4
/* Conversion arguments and results */
Boolean status;
Display *display;
XrmValue args[2];
Cardinal num_args;
XrmValue fromVal;
XrmValue toVal;
XtPointer *closure_ret = (XtPointer *) 0;
/* String to Visual specific */
Screen *screen;
Cardinal depth;
char *visstr;
Visual *res;
pid_t pid2;
#endif
pid_t	pid;

 	report_purpose(1);

	report_assertion("Assertion XtCvtStringToVisual-1.(C)");
	report_assertion("If the implementation is X11R5 or later: A successful call");
	report_assertion("to Boolean XtCvtStringToVisual(display, args, num_args,");
	report_assertion("from, to, converter_data) when to->addr is not NULL and");
	report_assertion("to->size is large enough to store an XtRVisual type shall");
	report_assertion("convert the string specified in from to an XtRVisual type,");
	report_assertion("store it at the location specified by to->addr, set to->size");
	report_assertion("to the actual size of the converted data, and return True.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

#if XT_X_RELEASE > 4
	FORK(pid2);
	avs_xt_hier("Tcstvis1", "XtCvtStringToVisual");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get display");
	display = XtDisplay(topLevel);
	tet_infoline("PREP: Set up required conversion args");
	screen = DefaultScreenOfDisplay(display);
	args[0].addr = (XtPointer )&screen;
	args[0].size = sizeof(Screen*);
	depth = DefaultDepthOfScreen(screen);
	args[1].addr = (XtPointer) &depth;
	args[1].size = sizeof(Cardinal);
	num_args = 2;
	tet_infoline("PREP: Choose a visual that exists");
	res = DefaultVisualOfScreen(screen);
	switch(res->class) {
	case StaticGray:
		visstr = "StaticGray";
		break;
	case GrayScale:
		visstr = "GrayScale";
		break;
	case StaticColor:
		visstr = "StaticColor";
		break;
	case PseudoColor:
		visstr = "PseudoColor";
		break;
	case TrueColor:
		visstr = "TrueColor";
		break;
	case DirectColor:
		visstr = "DirectColor";
		break;
	}
	tet_infoline("TEST: Check conversion succeeds");
	fromVal.addr = visstr;
	fromVal.size = strlen(visstr)+1;
	toVal.addr = (XtPointer) &res;
	toVal.size = sizeof(XtPointer);
	status = XtCvtStringToVisual(display,
			&args[0],
			&num_args,
			&fromVal,
			&toVal,
			closure_ret);
	check_dec(True, status, "XtCvtStringToVisual return value");
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_result(TET_PASS);
#else
	tet_infoline("INFO: Implementation not X11R5 or greater");
	tet_result(TET_UNSUPPORTED);
#endif

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11693 */
/* End of included file Mdefcode.tmc.11693 */

static void t002(){

#if XT_X_RELEASE > 4
/* Conversion arguments and results */
Boolean status;
Display *display;
XrmValue args[2];
Cardinal num_args;
XrmValue fromVal;
XrmValue toVal;
XtPointer *closure_ret = (XtPointer *) 0;
/* String to Visual specific */
Screen *screen;
Cardinal depth;
char *visstr = "fixed";
Visual *res;
pid_t pid2;
#endif
pid_t	pid;

 	report_purpose(2);

	report_assertion("Assertion XtCvtStringToVisual-2.(C)");
	report_assertion("If the implementation is X11R5 or later: A call to Boolean");
	report_assertion("XtCvtStringToVisual(display, args, num_args, from, to,");
	report_assertion("converter_data) when to->addr is not NULL and to->size is");
	report_assertion("too small for an XtRVisual type shall not perform a");
	report_assertion("conversion, set the to->size field to the number of bytes");
	report_assertion("required to store the converted data, and return False.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

#if XT_X_RELEASE > 4
	FORK(pid2);
	avs_xt_hier("Tcstvis2", "XtCvtStringToVisual");
	XtAppSetWarningMsgHandler(app_ctext, XtWMH_Proc);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get display");
	display = XtDisplay(topLevel);
	tet_infoline("PREP: Set up required conversion args");
	screen = DefaultScreenOfDisplay(display);
	args[0].addr = (XtPointer )&screen;
	args[0].size = sizeof(Screen*);
	depth = DefaultDepthOfScreen(screen);
	args[1].addr = (XtPointer) &depth;
	args[1].size = sizeof(Cardinal);
	num_args = 2;
	tet_infoline("PREP: Choose a visual that exists");
	res = DefaultVisualOfScreen(screen);
	switch(res->class) {
	case StaticGray:
		visstr = "StaticGray";
		break;
	case GrayScale:
		visstr = "GrayScale";
		break;
	case StaticColor:
		visstr = "StaticColor";
		break;
	case PseudoColor:
		visstr = "PseudoColor";
		break;
	case TrueColor:
		visstr = "TrueColor";
		break;
	case DirectColor:
		visstr = "DirectColor";
		break;
	}
	tet_infoline("TEST: Check conversion fails");
	fromVal.addr = visstr;
	fromVal.size = strlen(visstr)+1;
	toVal.addr = (XtPointer) &res;
	toVal.size = 0;
	status = XtCvtStringToVisual(display,
			&args[0],
			&num_args,
			&fromVal,
			&toVal,
			closure_ret);
	check_dec(False, status, "XtCvtStringToVisual return value");
	tet_infoline("TEST: to_size");
	if (toVal.size != sizeof(Visual *)) {
		sprintf(ebuf, "ERROR: to_size not set correctly, expected %d, received %d", sizeof(Visual *), toVal.size);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_result(TET_PASS);
#else
	tet_infoline("INFO: Implementation not X11R5 or greater");
	tet_result(TET_UNSUPPORTED);
#endif

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11693 */
/* End of included file Mdefcode.tmc.11693 */

static void t003(){

#if XT_X_RELEASE > 4
/* Conversion arguments and results */
Boolean status;
Display *display;
XrmValue args[2];
Cardinal num_args;
XrmValue fromVal;
XrmValue toVal;
XtPointer *closure_ret = (XtPointer *) 0;
/* String to Visual specific */
Screen *screen;
Cardinal depth;
char *visstr = "fixed";
Visual *res;
pid_t pid2;
#endif
pid_t	pid;

 	report_purpose(3);

	report_assertion("Assertion XtCvtStringToVisual-3.(C)");
	report_assertion("If the implementation is X11R5 or later: A successful call");
	report_assertion("to Boolean XtCvtStringToVisual(display, args, num_args,");
	report_assertion("from, to, converter_data) when to->addr is NULL shall");
	report_assertion("convert the string specified in from to an XtRVisual type,");
	report_assertion("allocate space for the converted data, set to->addr to");
	report_assertion("specify the location of the data, set to->size to the size");
	report_assertion("of the coverted data, and return True.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

#if XT_X_RELEASE > 4
	FORK(pid2);
	avs_xt_hier("Tcstvis3", "XtCvtStringToVisual");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get display");
	display = XtDisplay(topLevel);
	tet_infoline("PREP: Set up required conversion args");
	screen = DefaultScreenOfDisplay(display);
	args[0].addr = (XtPointer )&screen;
	args[0].size = sizeof(Screen*);
	depth = DefaultDepthOfScreen(screen);
	args[1].addr = (XtPointer) &depth;
	args[1].size = sizeof(Cardinal);
	num_args = 2;
	tet_infoline("PREP: Choose a visual that exists");
	res = DefaultVisualOfScreen(screen);
	switch(res->class) {
	case StaticGray:
		visstr = "StaticGray";
		break;
	case GrayScale:
		visstr = "GrayScale";
		break;
	case StaticColor:
		visstr = "StaticColor";
		break;
	case PseudoColor:
		visstr = "PseudoColor";
		break;
	case TrueColor:
		visstr = "TrueColor";
		break;
	case DirectColor:
		visstr = "DirectColor";
		break;
	}
	tet_infoline("TEST: Check conversion succeeds");
	fromVal.addr = visstr;
	fromVal.size = strlen(visstr)+1;
	toVal.addr = (XtPointer)0;
	toVal.size = 0;
	status = XtCvtStringToVisual(display,
			&args[0],
			&num_args,
			&fromVal,
			&toVal,
			closure_ret);
	check_dec(True, status, "XtCvtStringToVisual return value");
	tet_infoline("TEST: to->addr and to->size are set");
	if (!toVal.addr) {
		sprintf(ebuf, "ERROR: to->addr was not set");
	tet_infoline(ebuf);
	tet_result(TET_FAIL);
	}
	if (toVal.size != sizeof(Visual *)) {
		sprintf(ebuf, "ERROR: to_size not set correctly, expected %d, received %d", sizeof(Visual *), toVal.size);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_result(TET_PASS);
#else
	tet_infoline("INFO: Implementation not X11R5 or greater");
	tet_result(TET_UNSUPPORTED);
#endif

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11693 */
/* End of included file Mdefcode.tmc.11693 */

static void t004(){

#if XT_X_RELEASE > 4
/* Conversion arguments and results */
Boolean status;
Display *display;
XrmValue args[2];
Cardinal num_args;
XrmValue fromVal;
XrmValue toVal;
XtPointer *closure_ret = (XtPointer *) 0;
/* String to Visual specific */
Screen *screen;
char *visstr = "fixed";
Visual *res;
pid_t pid2;
#endif
pid_t	pid;

 	report_purpose(4);

	report_assertion("Assertion XtCvtStringToVisual-4.(C)");
	report_assertion("If the implementation is X11R5 or later: A call to Boolean");
	report_assertion("XtCvtStringToVisual(display, args, num_args, from, to,");
	report_assertion("converter_data) when args specifies an improper value shall");
	report_assertion("not perform a conversion, issue a warning message, and");
	report_assertion("return False.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

#if XT_X_RELEASE > 4
	avs_set_event(1,0);
	FORK(pid2);
	avs_xt_hier("Tcstvis4", "XtCvtStringToVisual");
	XtAppSetWarningMsgHandler(app_ctext, XtWMH_Proc);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get display");
	display = XtDisplay(topLevel);
	tet_infoline("PREP: Set up required conversion args incorrectly");
	num_args = 0;
	tet_infoline("PREP: Choose a visual that exists");
	screen = DefaultScreenOfDisplay(display);
	res = DefaultVisualOfScreen(screen);
	switch(res->class) {
	case StaticGray:
		visstr = "StaticGray";
		break;
	case GrayScale:
		visstr = "GrayScale";
		break;
	case StaticColor:
		visstr = "StaticColor";
		break;
	case PseudoColor:
		visstr = "PseudoColor";
		break;
	case TrueColor:
		visstr = "TrueColor";
		break;
	case DirectColor:
		visstr = "DirectColor";
		break;
	}
	tet_infoline("TEST: Check conversion fails");
	fromVal.addr = visstr;
	fromVal.size = strlen(visstr)+1;
	toVal.addr = (XtPointer)0;
	toVal.size = 0;
	status = XtCvtStringToVisual(display,
			&args[0],
			&num_args,
			&fromVal,
			&toVal,
			closure_ret);
	check_dec(False, status, "XtCvtStringToVisual return value");
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Warning handler is called");
	status = avs_get_event(1);
	check_dec(1, status, "calls to warning handler count");
	tet_result(TET_PASS);
#else
	tet_infoline("INFO: Implementation not X11R5 or greater");
	tet_result(TET_UNSUPPORTED);
#endif

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11693 */
/* End of included file Mdefcode.tmc.11693 */

static void t005(){

#if XT_X_RELEASE > 4
/* Conversion arguments and results */
Boolean status;
Display *display;
XrmValue args[2];
Cardinal num_args;
XrmValue fromVal;
XrmValue toVal;
XtPointer *closure_ret = (XtPointer *) 0;
/* String to Visual specific */
Screen *screen;
Cardinal depth;
char *visstr="Visual from mars";
Visual *res;
pid_t pid2;
#endif
pid_t	pid;

 	report_purpose(5);

	report_assertion("Assertion XtCvtStringToVisual-5.(A)");
	report_assertion("If the implementation is X11R5 or later: A call to Boolean");
	report_assertion("XtCvtStringToVisual(display, args, num_args, from, to,");
	report_assertion("converter_data) when the string specified by from is not");
	report_assertion("StaticGray, StaticColor, TrueColor, GrayScale, PseudoColor");
	report_assertion("or DirectColor shall issue a warning message and return");
	report_assertion("False.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

#if XT_X_RELEASE > 4
	FORK(pid2);
	avs_set_event(2, 0);
	avs_xt_hier("Tcstvis5", "XtCvtStringToVisual");
	XtAppSetWarningMsgHandler(app_ctext, XtWMH_Proc);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get display");
	display = XtDisplay(topLevel);
	tet_infoline("PREP: Set up required conversion args");
	screen = DefaultScreenOfDisplay(display);
	args[0].addr = (XtPointer )&screen;
	args[0].size = sizeof(Screen*);
	depth = DefaultDepthOfScreen(screen);
	args[1].addr = (XtPointer) &depth;
	args[1].size = sizeof(Cardinal);
	num_args = 2;
	tet_infoline("TEST: Check conversion fails");
	fromVal.addr = visstr;
	fromVal.size = strlen(visstr)+1;
	toVal.addr = (XtPointer) &res;
	toVal.size = sizeof(XtPointer);
	status = XtCvtStringToVisual(display,
			&args[0],
			&num_args,
			&fromVal,
			&toVal,
			closure_ret);
	check_dec(False, status, "XtCvtStringToVisual return value");
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Warning handler is called");
	status = avs_get_event(1);
	check_dec(1, status, "calls to warning handler count");
	tet_result(TET_PASS);
#else
	tet_infoline("INFO: Implementation not X11R5 or greater");
	tet_result(TET_UNSUPPORTED);
#endif

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11693 */
/* End of included file Mdefcode.tmc.11693 */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ t002, 2 },
	{ t003, 3 },
	{ t004, 4 },
	{ t005, 5 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = NULL;
void	(*tet_cleanup)() = NULL;
