/*
* Copyright (c) 2005 X.Org Foundation L.L.C.
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) 2001 The Open Group
* Copyright (c) Applied Testing and Technology, Inc. 1993, 1994, 1995
* Copyright (c) 88open Consortium, Ltd. 1990, 1991, 1992, 1993
* All Rights Reserved.
* 
*/
/*
 * SYNOPSIS:
 *   void
 *   XtAppGetErrorDatabaseText()
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <XtTest.h>

/*error messages created here*/
char	ebuf[4096];
/*allows a test to extend how long it runs*/
int	xt_tomultiple = 1;

#include <X11/IntrinsicP.h>
#include <X11/ConstrainP.h>
#include <X11/CoreP.h>

#define ER_SIZE    512
char file_name[1024] ;

XtAppContext app_ctext;
Widget topLevel, panedw, boxw1, boxw2;
Widget labelw, rowcolw, click_quit;


#define T_XtAppGetErrorDatabaseText	1
char    *TestName = "XtAppGetErrorDatabaseText";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

XrmDatabase *database, file_database, db_return;
char buffer[ER_SIZE];
char *contents = "testString.error:Testing XtErrorMsg\n";
char *ErrorString = "Testing XtErrorMsg";
pid_t pid2;
pid_t	pid;

 	report_purpose(1);

	report_assertion("Assertion XtAppGetErrorDatabaseText-1.(A)");
	report_assertion("A successful call to void");
	report_assertion("XtAppGetErrorDatabaseText(app_context, name, type, class,");
	report_assertion("default, buffer_return, nbytes, database) when database is");
	report_assertion("not NULL shall return the error message in buffer_return for");
	report_assertion("the error specified by the resource name name.type, resource");
	report_assertion("class class.class in the error database specified by");
	report_assertion("database.");
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Tapgterdt1", "XtAppGetErrorDatabaseText");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get the file name to dump error message");
	strcpy(file_name, "/tmp/");
	strcat(file_name, "outfile");
	tet_infoline("PREP: Create error database");
	XrmInitialize();
	db_return = XrmGetStringDatabase(contents);
	(void)XrmPutFileDatabase(db_return,file_name);
	file_database = XrmGetFileDatabase(file_name);
	tet_infoline("TEST: Get message for testString.error");
	XtAppGetErrorDatabaseText(app_ctext,
		"testString", "error", "AppError",
		"ERROR: Unable find database Text ", buffer,
		 ER_SIZE, file_database);
	sprintf(ebuf, "TEST: Message was %s", ErrorString);
	tet_infoline(ebuf);
	check_str(ErrorString, buffer, "String");
	unlink(file_name);
	LKROF(pid2, AVSXTTIMEOUT-2);
	unlink(file_name);
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.12023 */
/* End of included file Mdefcode.tmc.12023 */

static void t002(){

XrmDatabase *database, file_database, db_return;
char buffer[ER_SIZE];
char *contents = "testString.error:Testing XtErrorMsg\n";
char *ErrorString = "Testing XtErrorMsg";
pid_t pid2;
pid_t	pid;

 	report_purpose(2);

	report_assertion("Assertion XtAppGetErrorDatabaseText-2.(A)");
	report_assertion("A call to void XtAppGetErrorDatabaseText(app_context, name,");
	report_assertion("type, class, default, buffer_return, nbytes, database) when");
	report_assertion("database is NULL shall return the error message in the");
	report_assertion("buffer buffer_return for the error specified by the resource");
	report_assertion("name name.type, resource class class.class in the error");
	report_assertion("database of the application context app_context.");
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Tapgterdt1", "XtAppGetErrorDatabaseText");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get the file name to dump error message");
	strcpy(file_name, "/tmp/");
	strcat(file_name, "outfile");
	tet_infoline("PREP: Create error database");
	XrmInitialize();
	db_return = XrmGetStringDatabase(contents);
	(void)XrmPutFileDatabase(db_return,file_name);
	file_database = XrmGetFileDatabase(file_name);
	tet_infoline("PREP: Obtain the error database");
	database = XtAppGetErrorDatabase(app_ctext);
	if (database == (XrmDatabase *)NULL) {
		sprintf(ebuf, "ERROR: Expected database to be non-NULL");
		tet_infoline(ebuf);
		tet_result(TET_UNRESOLVED);
	}
	tet_infoline("PREP: Merge original database with new database");
	XrmMergeDatabases(file_database, database);
	tet_infoline("TEST: Get message for testString.error");
	XtAppGetErrorDatabaseText(app_ctext,
		"testString", "error", "AppError",
		"ERROR: Unable find database Text ", buffer,
		 ER_SIZE, (XrmDatabase) NULL);
	sprintf(ebuf, "TEST: Message was %s", ErrorString);
	tet_infoline(ebuf);
	check_str(ErrorString, buffer, "String");
	unlink(file_name);
	LKROF(pid2, AVSXTTIMEOUT-2);
	unlink(file_name);
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.12023 */
/* End of included file Mdefcode.tmc.12023 */

static void t003(){

pid_t pid2;
XrmDatabase *database, file_database, db_return;
char buffer[ER_SIZE];
char *contents = "testString.error:Testing XtErrorMsg\n";
char *ErrorString = "Testing XtErrorMsg";
pid_t	pid;

 	report_purpose(3);

	report_assertion("Assertion XtAppGetErrorDatabaseText-3.(A)");
	report_assertion("When database is not NULL and no entry exists in the error");
	report_assertion("database for for the error specified by the resource name");
	report_assertion("name.type, resource class class.class a call to void");
	report_assertion("XtAppGetErrorDatabaseText(app_context, name, type, class,");
	report_assertion("default, buffer_return, nbytes, database) shall return the");
	report_assertion("string specified by default in buffer_return.");
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Tapgterdt2", "XtAppGetErrorDatabaseText");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Get the file name to dump error message");
	strcpy(file_name, "/tmp/");
	strcat(file_name, "outfile");
	tet_infoline("PREP: Create error database");
	XrmInitialize();
	db_return = XrmGetStringDatabase(contents);
	(void)XrmPutFileDatabase(db_return,file_name);
	file_database = XrmGetFileDatabase(file_name);
	tet_infoline("TEST: Call XtAppGetErrorDatabaseText");
	XtAppGetErrorDatabaseText(app_ctext,
		"notThereString", "error", "AppError",
		"Hello World", buffer,
		 ER_SIZE, file_database);
	tet_infoline("TEST: Message was default");
	check_str("Hello World", buffer, "Default String");
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.12023 */
/* End of included file Mdefcode.tmc.12023 */

static void t004(){

char buffer[ER_SIZE];
pid_t pid2;
pid_t	pid;

 	report_purpose(4);

	report_assertion("Assertion XtAppGetErrorDatabaseText-4.(A)");
	report_assertion("When database is NULL and no entry exists in the error");
	report_assertion("database of the application context app_context for the");
	report_assertion("error specified by the resource name name.type, resource");
	report_assertion("class class.class a call to void");
	report_assertion("XtAppGetErrorDatabaseText(app_context, name, type, class,");
	report_assertion("default, buffer_return, nbytes, database) shall return the");
	report_assertion("string specified by default in buffer_return.");
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Tapgterdt2", "XtAppGetErrorDatabaseText");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("TEST: Call XtAppGetErrorDatabaseText");
	XtAppGetErrorDatabaseText(app_ctext,
		"testString", "error", "AppError",
		"Hello World", buffer,
		 ER_SIZE, (XrmDatabase)NULL);
	tet_infoline("TEST: Message was default");
	check_str("Hello World", buffer, "Default String");
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.12023 */
/* End of included file Mdefcode.tmc.12023 */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ t002, 2 },
	{ t003, 3 },
	{ t004, 4 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = NULL;
void	(*tet_cleanup)() = NULL;
