/*
* Copyright (c) 2005 X.Org Foundation LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) Applied Testing and Technology, Inc. 1993, 1994, 1995
* Copyright (c) 88open Consortium, Ltd. 1990, 1991, 1992, 1993
* All Rights Reserved.
* 
*/
/*
 * SYNOPSIS:
 *   Screen *
 *   XtScreenOfObject()
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <XtTest.h>

/*error messages created here*/
char	ebuf[4096];
/*allows a test to extend how long it runs*/
int	xt_tomultiple = 1;

#include <X11/IntrinsicP.h>
#include <X11/ConstrainP.h>
#include <X11/CoreP.h>

XtAppContext app_ctext;
Widget topLevel, panedw, boxw1, boxw2;
Widget labelw, rowcolw, click_quit;


#define T_XtScreenOfObject	1
char    *TestName = "XtScreenOfObject";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

Screen *screen_good, *ret_screen;
Display *display_good;
int ret_screen_id , screen_id_good;
pid_t	pid;

 	report_purpose(1);

	report_assertion("Assertion XtScreenOfObject-1.(A)");
	report_assertion("A call to Screen *XtScreenOfObject(object) when object is a");
	report_assertion("widget shall return the screen pointer for the widget.");
	FORK(pid);
	initconfig();

	avs_xt_hier("Tscnofobt1", "XtScreenOfObject");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("TEST: Get the screen pointer for widget label");
	if ((screen_good = XtScreenOfObject(labelw)) == NULL) {
			sprintf(ebuf, "ERROR: Expected Screen pointer returned NULL");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_infoline("TEST: Returned screen pointer is valid");
	display_good = XtDisplay(labelw);
	ret_screen = XDefaultScreenOfDisplay(display_good);
	ret_screen_id = XScreenNumberOfScreen(ret_screen);
	screen_id_good = XScreenNumberOfScreen(screen_good);
	check_dec(screen_id_good, ret_screen_id, "screen number");
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.10946 */
/* End of included file Mdefcode.tmc.10946 */

static void t002(){

Screen *screen_good, *ret_screen;
Display *display_good;
int ret_screen_id , screen_id_good;
pid_t	pid;

 	report_purpose(2);

	report_assertion("Assertion XtScreenOfObject-2.(A)");
	report_assertion("A call to Screen *XtScreenOfObject(object) when object is");
	report_assertion("not a widget shall return the screen pointer for the nearest");
	report_assertion("ancestor of object that is of class Object or a subclass");
	report_assertion("thereof.");
	FORK(pid);
	initconfig();

	avs_xt_hier("Tscnofobt2", "XtScreenOfObject");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("TEST: Get the screen pointer for push button gadget");
	if ((screen_good = XtScreenOfObject(click_quit)) == NULL) {
		sprintf(ebuf, "ERROR: Expected Screen pointer returned NULL");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_infoline("TEST: Returned screen pointer is that of the gadget's parent");
	display_good = XtDisplay(rowcolw);
	ret_screen = XDefaultScreenOfDisplay(display_good);
	ret_screen_id = XScreenNumberOfScreen(ret_screen);
	screen_id_good = XScreenNumberOfScreen(screen_good);
	check_dec(screen_id_good, ret_screen_id, "screen number");
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.10946 */
/* End of included file Mdefcode.tmc.10946 */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ t002, 2 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = NULL;
void	(*tet_cleanup)() = NULL;
