/*
* Copyright (c) 2005 X.Org Foundation LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) Applied Testing and Technology, Inc. 1993, 1994, 1995
* Copyright (c) 88open Consortium, Ltd. 1990, 1991, 1992, 1993
* All Rights Reserved.
* 
*/
/*
 * SYNOPSIS:
 *   Boolean
 *   XtIsManaged()
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <XtTest.h>

/*error messages created here*/
char	ebuf[4096];
/*allows a test to extend how long it runs*/
int	xt_tomultiple = 1;

#include <X11/IntrinsicP.h>
#include <X11/ConstrainP.h>
#include <X11/CoreP.h>
#include <X11/Xaw/Label.h>
#include <AvsObj.h>

XtAppContext app_ctext;
Widget topLevel, panedw, boxw1, boxw2;
Widget labelw, rowcolw, click_quit;


#define T_XtIsManaged	1
char    *TestName = "XtIsManaged";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

Boolean status;
pid_t	pid;

 	report_purpose(1);

	report_assertion("Assertion XtIsManaged-1.(A)");
	report_assertion("A successful call to Boolean XtIsManaged(w) when the widget");
	report_assertion("w belongs to the class RectObj or a subclass thereof and is");
	report_assertion("managed shall return True.");
	FORK(pid);
	initconfig();

	avs_xt_hier("Tismanage1", "XtIsManaged");
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("TEST: labelw widget is managed shall return True");
	status = XtIsManaged(labelw);
	check_dec(True, status, "XtIsManaged return value");
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.11075 */
/* End of included file Mdefcode.tmc.11075 */

static void t002(){

Widget unmngw;
Boolean status;
pid_t	pid;

 	report_purpose(2);

	report_assertion("Assertion XtIsManaged-2.(A)");
	report_assertion("A call to Boolean XtIsManaged(w) when the widget w is not");
	report_assertion("managed shall return False.");
	FORK(pid);
	initconfig();

	avs_xt_hier("Tismanage2", "XtIsManaged");
	tet_infoline("PREP: Create unmngw label widget in box widget");
	unmngw = XtVaCreateWidget(
		"Unmanaged Widget",
		labelWidgetClass,
		boxw1,
		NULL
	);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("TEST: XtIsManaged for unmngw label widget returns False");
	status = XtIsManaged(unmngw);
	check_dec(False, status, "XtIsManaged return value");
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.11075 */
/* End of included file Mdefcode.tmc.11075 */

static void t003(){

Widget ObjectNotWidget;
Boolean status;
pid_t	pid;

 	report_purpose(3);

	report_assertion("Assertion XtIsManaged-3.(A)");
	report_assertion("A call to Boolean XtIsManaged(w) when the widget w does not");
	report_assertion("belong to the class RectObj or a subclass thereof shall");
	report_assertion("return False.");
	FORK(pid);
	initconfig();

	avs_xt_hier("Tismanage2", "XtIsManaged");
	tet_infoline("PREP: Create an AvsObject");
	ObjectNotWidget = XtCreateWidget("ObjectNotWidget", avsObjClass,
			 topLevel, (ArgList) 0, 0);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("TEST: XtIsManaged Returns False");
	status = XtIsManaged(ObjectNotWidget) ;
	check_dec(False, status, "Return value");
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
}

/* Including from file Mdefcode.tmc.11075 */
/* End of included file Mdefcode.tmc.11075 */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ t002, 2 },
	{ t003, 3 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = NULL;
void	(*tet_cleanup)() = NULL;
