/*
* Copyright (c) 2005 X.Org Foundation LLC
* 
* Permission is hereby granted, free of charge, to any person obtaining a copy of
* this software and associated documentation files (the "Software"), to deal in
* the Software without restriction, including without limitation the rights to
* use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
* of the Software, and to permit persons to whom the Software is furnished to do
* so, subject to the following conditions:
* 
* The above copyright notice and this permission notice shall be included in all
* copies or substantial portions of the Software.
* 
* THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
* AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
* LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
* OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
* SOFTWARE.
* 
* Copyright (c) Applied Testing and Technology, Inc. 1993, 1994, 1995
* Copyright (c) 88open Consortium, Ltd. 1990, 1991, 1992, 1993
* All Rights Reserved.
* 
*/
/*
 * SYNOPSIS:
 *   void
 *   XtRemoveEventHandler()
 */


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <XtTest.h>

/*error messages created here*/
char	ebuf[4096];
/*allows a test to extend how long it runs*/
int	xt_tomultiple = 1;

#include <X11/IntrinsicP.h>
#include <X11/ConstrainP.h>
#include <X11/CoreP.h>

XtAppContext app_ctext;
Widget topLevel, panedw, boxw1, boxw2;
Widget labelw, rowcolw, click_quit;

extern char *event_names[];

/*
** Procedure XtEVT1_Proc
*/
void XtEVT1_Proc(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	if (event->type == ButtonPress) {
		avs_set_event(1,1);
		return;
	}
	if (event->type == KeyPress) {
		avs_set_event(2,1);
		return;
	}
	if (event->type == ButtonRelease) {
		avs_set_event(3,1);
		return;
	}
	if (event->type == KeyRelease) {
		avs_set_event(4,1);
		return;
	}

	sprintf(ebuf, "ERROR: Expected Button or Key event, Received %s", event_names[event->type]);
	tet_infoline(ebuf);
	tet_result(TET_FAIL);
	exit(0);
}
/* procedure XtTMO1_Proc to be invoked */
void XtTMO1_Proc(client_data, id)
XtPointer client_data;
XtIntervalId *id;
{
	exit(0);
}

void XtEVT3_Test(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	sprintf(ebuf, "ERROR: Event %s invoked XtEVT3_Test", event_names[event->type]);
	tet_infoline(ebuf);
	tet_result(TET_FAIL);
}
/*
** Procedure XtEVT3_Proc
*/
void XtEVT3_Proc(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	if ( event->type == ButtonPress )
	avs_set_event(1,1);
	else {
		sprintf(ebuf, "ERROR: Expected ButtonPress event Received %s", event_names[event->type]);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	exit(0);
}

/*
** Procedure XtEVT2_Proc
*/
void XtEVT2_Proc(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	if ( event->type == ButtonPress )
	avs_set_event(1,1);
	else {
		sprintf(ebuf, "ERROR: Expected ButtonPress event Received %s", event_names[event->type]);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	exit(0);
}
/*
** Procedure XtEVT4_Proc
*/
void XtEVT4_Proc(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	if ((event->type == ButtonPress) || (event->type == KeyPress))
		avs_set_event(1, avs_get_event(1)+1);
	else {
		sprintf(ebuf, "ERROR: Expected ButtonPress and KeyPress events, Received %s", event_names[event->type]);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
}
/*
** Procedure XtEVT4a_Proc
*/
void XtEVT4a_Proc(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	if ((event->type == ButtonRelease) || (event->type == KeyRelease))
		avs_set_event(2, avs_get_event(2)+1);
	else {
		sprintf(ebuf, "ERROR: Expected ButtonPress and KeyPress events, Received %s", event_names[event->type]);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
}
/* procedure XtTMO4_Proc to be invoked */
void XtTMO4_Proc(client_data, id)
XtPointer client_data;
XtIntervalId *id;
{
	exit(0);
}
/*
** Warning handler
*/
void XtWMH_Proc(str1, str2, str3, str4, str5, car)
String str1, str2, str3, str4, *str5;
Cardinal *car;
{
	avs_set_event(2,1);
}
/*
** Error handler
*/
void XtEMH_Proc(str1, str2, str3, str4, str5, car)
String str1, str2, str3, str4, *str5;
Cardinal *car;
{
	avs_set_event(3,1);
}
void XtEVT_ProcA(w, client_data, event, continue_to_dispatch)
Widget w;
XtPointer client_data;
XEvent *event;
Boolean *continue_to_dispatch;
{
	if (event->type == GraphicsExpose )
	avs_set_event(1,1);
	else {
		sprintf(ebuf, "ERROR: Expected GraphicsExpose event Received %s", event_names[event->type]);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	exit(0);
}


#define T_XtRemoveEventHandler	1
char    *TestName = "XtRemoveEventHandler";



int 	tet_thistest;

/*
 * Set the arguments to default values for error tests
 */
static void
seterrdef()
{
}

static void t001(){

Widget labelw_msg;
char *msg = "Event widget";
pid_t pid2;
int invoked = 0;
pid_t	pid;

 	report_purpose(1);

	report_assertion("Assertion XtRemoveEventHandler-1.(A)");
	report_assertion("When proc and client_data match a handler/data pair");
	report_assertion("previously registered for the widget w by a call to");
	report_assertion("XtAddEventHandler or XtInsertEventHandler a successful call");
	report_assertion("to void XtRemoveEventHandler(w, event_mask, nonmaskable,");
	report_assertion("proc, client_data) shall unregister the procedure for the");
	report_assertion("events specified by event_mask.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Trmevthdr1", "XtRemoveEventHandler");
	tet_infoline("PREP: Create a test labelw_msg widget");
	labelw_msg = (Widget) CreateLabelWidget(msg, boxw1);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Register XtEVT1_Proc to handle ButtonPress events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Register XtEVT1_Proc to handle KeyPress events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 KeyPressMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Insert XtEVT1_Proc to handle ButtonRelease events to labelw_msg");
	XtInsertEventHandler(labelw_msg,
		 ButtonReleaseMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL,
		 XtListHead
		 );
	tet_infoline("PREP: Insert XtEVT1_Proc to handle KeyRelease events to labelw_msg");
	XtInsertEventHandler(labelw_msg,
		 KeyReleaseMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL,
		 XtListTail
		 );
	tet_infoline("PREP: Remove XtEVT1_Proc for ButtonPress events");
	XtRemoveEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Remove XtEVT1_Proc for KeyRelease events");
	XtRemoveEventHandler(labelw_msg,
		 KeyReleaseMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Send ButtonPress event");
	send_event(labelw_msg, ButtonPress, ButtonPressMask, FALSE);
	tet_infoline("PREP: Send KeyPress event");
	send_event(labelw_msg, KeyPress, KeyPressMask, FALSE);
	tet_infoline("PREP: Send ButtonRelease event");
	send_event(labelw_msg, ButtonRelease, ButtonReleaseMask, FALSE);
	tet_infoline("PREP: Send KeyRelease event");
	send_event(labelw_msg, KeyRelease, KeyReleaseMask, FALSE);
	tet_infoline("PREP: Register timeout procedure");
	XtAppAddTimeOut(app_ctext, AVSXTLOOPTIMEOUT, XtTMO1_Proc, topLevel);
	XtAppMainLoop(app_ctext);
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Procedure XtEVT1_Proc was invoked only for KeyPress and ButtonRelease");
	if (avs_get_event(1) != 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was invoked for ButtonPress");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(2) == 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was not invoked for KeyPress");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(4) != 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was invoked for KeyRelease");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(3) == 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was not invoked for ButtonRelease");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11411 */
/* End of included file Mdefcode.tmc.11411 */

static void t002(){

Widget labelw_msg;
char *msg = "Event widget";
pid_t pid2;
int invoked = 0;
pid_t	pid;

 	report_purpose(2);

	report_assertion("Assertion XtRemoveEventHandler-2.(A)");
	report_assertion("When proc and client_data match a handler/data pair");
	report_assertion("previously registered for non-maskable events and");
	report_assertion("nonmaskable is True a successful call to void");
	report_assertion("XtRemoveEventHandler(w, event_mask, nonmaskable, proc,");
	report_assertion("client_data) shall unregister the procedure for all");
	report_assertion("non-maskable events.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Trmevthdr1", "XtRemoveEventHandler");
	tet_infoline("PREP: Create a test labelw_msg widget");
	labelw_msg = (Widget) CreateLabelWidget(msg, boxw1);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Register procedure to handle non-maskable events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 0,
		 True,
		 XtEVT_ProcA,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Remove procedure for nonmaskable events");
	XtRemoveEventHandler(labelw_msg,
		 0,
		 True,
		 XtEVT_ProcA,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Send GraphicsExpose event");
	send_event(labelw_msg, GraphicsExpose, 0, FALSE);
	XtAppAddTimeOut(app_ctext, AVSXTLOOPTIMEOUT, XtTMO1_Proc, topLevel);
	XtAppMainLoop(app_ctext);
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Procedure was not invoked");
	if (avs_get_event(1) != 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was invoked for GraphicsExpose");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11411 */
/* End of included file Mdefcode.tmc.11411 */

static void t003(){

Widget labelw_msg;
char *msg = "Event widget";
pid_t pid2;
int invoked = 0;
pid_t	pid;

 	report_purpose(3);

	report_assertion("Assertion XtRemoveEventHandler-3.(A)");
	report_assertion("When client_data does not match the value used when the");
	report_assertion("procedure proc was registered a call to void");
	report_assertion("XtRemoveEventHandler(w, event_mask, nonmaskable, proc,");
	report_assertion("client_data) shall return without performing any action.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Trmevthdr2", "XtRemoveEventHandler");
	tet_infoline("PREP: Create a test labelw_msg widget");
	labelw_msg = (Widget) CreateLabelWidget(msg, boxw1);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Register XtEVT2_Proc to handle events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT2_Proc,
		 (Widget)topLevel
		 );
	tet_infoline("PREP: Set up test XtToolkitError handler");
	XtAppSetErrorMsgHandler(app_ctext, XtEMH_Proc);
	tet_infoline("PREP: Set Warning Message Handler");
	XtAppSetWarningMsgHandler(app_ctext, XtWMH_Proc);
	tet_infoline("PREP: Remove the event handler with different client_data");
	XtRemoveEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT2_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Send event for which handler was registered");
	send_event(labelw_msg, ButtonPress, ButtonPressMask, FALSE);
	XtAppMainLoop(app_ctext);
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Error message was not emitted");
	if (avs_get_event(3) != 0) {
		sprintf(ebuf, "ERROR: Error message handler was invoked %d times");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(2) != 0) {
		sprintf(ebuf, "ERROR: Warning message handler was invoked %d times");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_infoline("TEST: Handler was invoked");
	invoked = avs_get_event(1);
	if (!invoked) {
		sprintf(ebuf, "ERROR: Handler was not invoked");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11411 */
/* End of included file Mdefcode.tmc.11411 */

static void t004(){

Widget labelw_msg;
char *msg = "Event widget";
pid_t pid2;
int invoked = 0;
pid_t	pid;

 	report_purpose(4);

	report_assertion("Assertion XtRemoveEventHandler-4.(A)");
	report_assertion("When the procedure proc has not been registered previously");
	report_assertion("for the specified widget a call to void");
	report_assertion("XtRemoveEventHandler(w, event_mask, nonmaskable, proc,");
	report_assertion("client_data) shall return without performing any action.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Trmevthdr3", "XtRemoveEventHandler");
	tet_infoline("PREP: Create a test labelw_msg widget");
	labelw_msg = (Widget) CreateLabelWidget(msg, boxw1);
	tet_infoline("PREP: Set up test XtToolkitError handler");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Register XtEVT3_Proc to handle events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT3_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Set up test XtToolkitError handler");
	XtAppSetErrorMsgHandler(app_ctext, XtEMH_Proc);
	tet_infoline("PREP: Set Warning Message Handler");
	XtAppSetWarningMsgHandler(app_ctext, XtWMH_Proc);
	tet_infoline("PREP: Remove a non-registered event handler");
	XtRemoveEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT3_Test,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Send event for which handler was registered");
	send_event(labelw_msg, ButtonPress, ButtonPressMask, FALSE);
	XtAppMainLoop(app_ctext);
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Handler was invoked");
	invoked = avs_get_event(1);
	if (!invoked) {
		sprintf(ebuf, "ERROR: Handler was not invoked");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_infoline("TEST: Error message was not emitted");
	if (avs_get_event(3) != 0) {
		sprintf(ebuf, "ERROR: Error message handler was invoked %d times");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(2) != 0) {
		sprintf(ebuf, "ERROR: Warning message handler was invoked %d times");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11411 */
/* End of included file Mdefcode.tmc.11411 */

static void t005(){

Widget labelw_msg;
char *msg = "Event widget";
pid_t pid2;
int invoked = 0;
pid_t	pid;

 	report_purpose(5);

	report_assertion("Assertion XtRemoveEventHandler-5.(A)");
	report_assertion("When event_mask is XtAllEvents and nonmaskable is True a");
	report_assertion("successful call to void XtRemoveEventHandler(w, event_mask,");
	report_assertion("nonmaskable, proc, client_data) shall cause the procedure");
	report_assertion("proc to be not called on any event for which it was");
	report_assertion("previously registered by calls to XtAddEventHandler or");
	report_assertion("XtInsertEventHandler.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Trmevthdr4", "XtRemoveEventHandler");
	tet_infoline("PREP: Create a test labelw_msg widget");
	labelw_msg = (Widget) CreateLabelWidget(msg, boxw1);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Register XtEVT4_Proc to handle events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 ButtonPressMask|KeyPressMask,
		 False,
		 XtEVT4_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Insert XtEVT4a_Proc to handle events to labelw_msg");
	XtInsertEventHandler(labelw_msg,
		 ButtonReleaseMask|KeyReleaseMask,
		 False,
		 XtEVT4_Proc,
		 (XtPointer)NULL,
		 XtListHead
		 );
	tet_infoline("PREP: Remove previously registered event handlers");
	XtRemoveEventHandler(labelw_msg,
		 XtAllEvents,
		 True,
		 XtEVT4a_Proc,
		 (XtPointer)NULL
		 );
	XtRemoveEventHandler(labelw_msg,
		 XtAllEvents,
		 True,
		 XtEVT4_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Register timeout procedure");
	XtAppAddTimeOut(app_ctext, AVSXTLOOPTIMEOUT, XtTMO4_Proc, topLevel);
	tet_infoline("TEST: Send all registered events for the two handlers");
	send_event(labelw_msg, ButtonPress, ButtonPressMask, FALSE);
	send_event(labelw_msg, KeyPress, KeyPressMask, FALSE);
	send_event(labelw_msg, ButtonRelease, ButtonReleaseMask, FALSE);
	send_event(labelw_msg, KeyRelease, KeyReleaseMask, FALSE);
	XtAppMainLoop(app_ctext);
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Handlers were not invoked");
	invoked = avs_get_event(1);
	if (invoked != 0) {
		sprintf(ebuf, "ERROR: XtEVT4_Proc was invoked %d times", invoked);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	invoked = avs_get_event(2);
	if (invoked != 0) {
		sprintf(ebuf, "ERROR:  XtEVT4a_Proc was invoked %d times", invoked);
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11411 */
/* End of included file Mdefcode.tmc.11411 */

static void t006(){

Widget labelw_msg;
char *msg = "Event widget";
pid_t pid2;
int invoked = 0;
pid_t	pid;

 	report_purpose(6);

	report_assertion("Assertion XtRemoveEventHandler-6.(A)");
	report_assertion("After a successful call to void XtRemoveEventHandler(w,");
	report_assertion("event_mask, nonmaskable, proc, client_data) when event_mask");
	report_assertion("is XtAllEvents and nonmaskable is True the procedure proc");
	report_assertion("shall continue to be called for those events for which it");
	report_assertion("was previously registered by calls to XtAddRawEventHandler");
	report_assertion("and XtInsertRawEventHandler.");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	FORK(pid2);
	avs_xt_hier("Trmevthdr5", "XtRemoveEventHandler");
	tet_infoline("PREP: Create a test labelw_msg widget");
	labelw_msg = (Widget) CreateLabelWidget(msg, boxw1);
	tet_infoline("PREP: Create windows for widgets and map them");
	XtRealizeWidget(topLevel);
	tet_infoline("PREP: Add XtEVT1_Proc to handle events to labelw_msg");
	XtAddEventHandler(labelw_msg,
		 ButtonPressMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Raw Add XtEVT1_Proc to handle events to labelw_msg");
	XtAddRawEventHandler(labelw_msg,
		 KeyReleaseMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Raw Insert XtEVT1_Proc to handle events to labelw_msg");
	XtInsertRawEventHandler(labelw_msg,
		 KeyPressMask,
		 False,
		 XtEVT1_Proc,
		 (XtPointer)NULL,
		 XtListHead
		 );
	tet_infoline("PREP: Unregister event handler for all events");
	XtRemoveEventHandler(labelw_msg,
		 XtAllEvents,
		 True,
		 XtEVT1_Proc,
		 (XtPointer)NULL
		 );
	tet_infoline("PREP: Register timeout procedure");
	XtAppAddTimeOut(app_ctext, AVSXTLOOPTIMEOUT, XtTMO4_Proc, topLevel);
	tet_infoline("TEST: Send all registered events to the widget");
	send_event(labelw_msg, ButtonPress, ButtonPressMask, FALSE);
	send_event(labelw_msg, KeyRelease, KeyReleaseMask, TRUE);
	send_event(labelw_msg, KeyPress, KeyPressMask, TRUE);
	XtAppMainLoop(app_ctext);
	LKROF(pid2, AVSXTTIMEOUT-2);
	tet_infoline("TEST: Handler was invoked but only for raw events");
	if (avs_get_event(1) != 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was invoked for ButtonPress");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(2) == 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was not invoked for KeyPress");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	if (avs_get_event(4) == 0) {
		sprintf(ebuf, "ERROR: XtEVT1_Proc was not invoked for KeyRelease");
		tet_infoline(ebuf);
		tet_result(TET_FAIL);
	}
	tet_result(TET_PASS);

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* Including from file Mdefcode.tmc.11411 */
/* End of included file Mdefcode.tmc.11411 */

/* Including from file Mdefcode.tmc.11411 */
static void t007(){

pid_t	pid;

 	report_purpose(7);

	report_assertion("Assertion XtRemoveEventHandler-7.(B)");
	report_assertion("When the widget w is realized a successful call to void");
	report_assertion("XtRemoveEventHandler(w, event_mask, nonmaskable, proc,");
	report_assertion("client_data) shall modify the event_mask attribute of the");
	report_assertion("widget window.");
	report_assertion("Reason for omission: There is no known portable test method for this assertion");

	avs_alloc_sem();
	FORK(pid);
	initconfig();

	UNTESTED;

	LKROF(pid, AVSXTTIMEOUT*xt_tomultiple);
	avs_free_sem();
}

/* End of included file Mdefcode.tmc.11411 */

/* End of Test Cases */


struct tet_testlist tet_testlist[] = {
	{ t001, 1 },
	{ t002, 2 },
	{ t003, 3 },
	{ t004, 4 },
	{ t005, 5 },
	{ t006, 6 },
	{ t007, 7 },
	{ NULL, 0 }
};

int 	ntests = sizeof(tet_testlist)/sizeof(struct tet_testlist)-1;

void	(*tet_startup)() = NULL;
void	(*tet_cleanup)() = NULL;
